// ステアリングコントローラーからの値の読み取りサンプル
// https://wlog.flatlib.jp/item/1682
// https://www.kernel.org/doc/Documentation/input/joystick-api.txt

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <linux/joystick.h>

#include <string.h>
#include <signal.h>
#include <errno.h>

#include "udp.h"

int main(int argc, char* argv[]) {
    if (argc != 3) {
        fprintf(stderr, "argument count mismatch error.\n");
        exit(EXIT_FAILURE);
    }

    /*** UDP Socketの初期化(ここから) ***/
    const char *address = argv[1];
    unsigned short port = (unsigned short)atoi(argv[2]);
    struct sockaddr servSockAddr, clitSockAddr;
    struct sigaction action;

    int server_sock = get_socket("udp");
    sockaddr_init(address, port, &servSockAddr);

    sigaction_init(&action, catchAlarm);
    if (sigaction(SIGALRM, &action, NULL) < 0) {
        perror("sigaction() failure");
        exit(EXIT_FAILURE);
    }
    /*** UDP Socketの初期化(ここまで) ***/

    // ステアリングコントローラーのデバイスファイルを開く
    int fd = open("/dev/input/js0", O_RDONLY);
    if (fd == -1) {
        printf("Can't open /dev/input/js0\n");
        exit(0);
    }

    int flag_hazard = 0; // 1(ON), 0(OFF)
    int flag_shift  = 0; // 3, 2, 1, 0(N), -1(R)
    int flag_winker = 0; // 1(R), 0(OFF), -1(L)
//    char flag_light  = 0; // 1, 2, 3, 4
    int flag_engine = 1; // 1(ON), 0(OFF)
    while (1) {
        struct js_event event;
        char sendBuffer[MAX_BUFSIZE];
        char receiveBuffer[MAX_BUFSIZE];

        double value = 0.0;

        // ステアリングコントローラーから読み取る
        if (read(fd, &event, sizeof(event))
                 < sizeof(event)) {
            continue;
        }

        switch (event.type & 0x7f) {
        case JS_EVENT_BUTTON:
            printf("BUTTON: number=%d, value=%d ",
                event.number, event.value);

            // ホーン(GTボタン)
            if (event.number == 19) {
                if (event.value == 1) {
                    udp_send(server_sock, "HORN=ON", &servSockAddr);
                    printf("HORN=ON");
                }
                else {
                    udp_send(server_sock, "HORN=OFF", &servSockAddr);
                    printf("HORN=OFF");
                }
            }
            // ハイビーム(○ボタン)
            else if (event.number == 2) {
                if (event.value == 1) {
                    udp_send(server_sock, "HIGHBEAM=ON", &servSockAddr);
                    printf("HIGHBEAM=ON");
                }
                else {
                    udp_send(server_sock, "HIGHBEAM=OFF", &servSockAddr);
                    printf("HIGHBEAM=OFF");
                }
            }
            // クラッチ
            // シフトチェンジ(パドルまたはレバー)
            else if ((event.number == 4 || event.number == 12 )
                    && event.value == 1) {
                udp_send(server_sock, "SHIFT=UP", &servSockAddr);
                printf("SHIFT=UP");
            }
            else if ((event.number == 5 || event.number == 13 )
                    && event.value == 1) {
                udp_send(server_sock, "SHIFT=DOWN", &servSockAddr);
                printf("SHIFT=DOWN");
            }
            // ウィンカー(R2,L2) ※改善の余地あり
            // ウィンカー直後はステアリングをキャンセルしたい…
            else if (event.number == 6 && event.value == 1) {
                flag_hazard = 0;
                udp_send(server_sock, "WINKER=RIGHT", &servSockAddr);
                printf("WINKER=RIGHT");
            }
            else if (event.number == 7 && event.value == 1) {
                flag_hazard = 0;
                udp_send(server_sock, "WINKER=LEFT", &servSockAddr);
                printf("WINKER=LEFT");
            }
            // ライト点灯(□)
            else if (event.number == 1 && event.value == 1) {
                udp_send(server_sock, "LIGHT=UP", &servSockAddr);
                printf("LIGHT=UP");
            }
            // ハザード(△)
            else if (event.number == 3 && event.value == 1) {
                flag_hazard ^= 1;
                if (flag_hazard) {
                    udp_send(server_sock, "HAZARD=ON", &servSockAddr);
                    printf("HAZARD=ON");
                }
                else {
                    udp_send(server_sock, "HAZARD=OFF", &servSockAddr);
                    printf("HAZARD=OFF");
                }
            }
            // エンジン停止・始動(PSボタン)
            else if (event.number == 20 && event.value == 1) {
                flag_engine ^= 1;
                if (flag_engine) {
                    udp_send(server_sock, "ENGINE=START", &servSockAddr);
                    printf("ENGINE=START");
                }
                else {
                    udp_send(server_sock, "ENGINE=STOP", &servSockAddr);
                    printf("ENGINE=STOP");
                }
            }

            printf("\n");
            break;

        case JS_EVENT_AXIS:
            printf("AXIS: number=%d, value=%d   ",
                event.number, event.value);
            // ステアリング
            if (event.number == 0) {
                value = event.value / 32767.0;
                sprintf(sendBuffer, "STEERING=%f", value);
                udp_send(server_sock, sendBuffer, &servSockAddr);
                printf(sendBuffer);
            }
            // アクセル(とりあえず前進だけ)
            else if (event.number == 1) {
                value = (32767.0 - event.value) / (32767*2);
                sprintf(sendBuffer, "ACCEL=%f", value);
                udp_send(server_sock, sendBuffer, &servSockAddr);
                printf(sendBuffer);
            }
            // ブレーキ
            else if (event.number == 2) {
                value = (32767.0 - event.value) / (32767*2);
                sprintf(sendBuffer, "BRAKE=%f", value);
                udp_send(server_sock, sendBuffer, &servSockAddr);
                printf(sendBuffer);
            }
            printf("\n");
            break;

        default:
            printf("OTHER\n");
            break;
        }
    }

    // 終了処理
    close(fd);
    return 0;
}

